rm(list=ls()) ; graphics.off() ; gc()

#Load libraries
library(ncdf4)
library(ggplot2)
library(gridExtra)
library(fields)
library(mapdata)
library(matrixStats)
library(metR)
library(RColorBrewer)
library(fields)
library(metR)
#library(transformeR)
#library(loadeR)
#library(climate4R.value)
#library(visualizeR)
#library(trend)
library(plotrix)
#library(miceadds)

rm(list=ls()) ; gc()

#Directory information
setwd("/esarchive/scratch/molmo/")
OUTPUTS <- "/esarchive/scratch/molmo/output/"

#Useful functions
png.save <- function(x,filename,main,xlab,ylab){
  png(filename = paste(filename,".png",sep=""))
  plot(x,type="o",main=main,xlab = xlab,ylab=ylab)
  dev.off()
}

avgmean <- function(x,ventana){
  value <- rep(NA,length(x))
  for(i in (1+ventana):(length(x)-ventana)){
    value[i] <- mean(x[(i-ventana):(i+ventana)],na.rm=T)
  }
  return(value)
}
####################################
#Load ERA5 data
message("Loading and preprocessing data...")
nc <- nc_open("psl_5022_50km_1x1.nc")
nc_lon <- ncvar_get(nc,"lon")
nc_lat <- ncvar_get(nc,"lat")
nc_var <- ncvar_get(nc,"psl") #x,y,time

#Selection of spatial domains
gridsize_x <- abs(nc_lon[2]-nc_lon[1])
gridsize_y <- abs(nc_lat[2]-nc_lat[1])

# domain1 <- list(lon <- which(nc_lon %in% seq(-15,15,by=gridsize_x)),
#                 lat <- which(nc_lat %in% seq(32,48,by=gridsize_y))) #red domain D1 #Spain
# 
# domain2 <- list(lon <- which(nc_lon %in% seq(-10,25,by=gridsize_x)),
#                 lat <- which(nc_lat %in% seq(33,45,by=gridsize_y))) #red domain D2 PEP Mediterranean
# 
# domain3 <- list(lon <- which(nc_lon %in% seq(-35,35,by=gridsize_x)),
#                 lat <- which(nc_lat %in% seq(20,70,by=gridsize_y))) #red domain D3 #Europe
#   patronGCM <- list()
# domain <- domain2 

#Organize and Standardize data
var <- matrix(nc_var,dim(nc_var)[1]*dim(nc_var)[2],dim(nc_var)[3]) #space,time
var <- scale(t(var),center=T,scale=T)
rm(nc_var) ; gc()

#Load clustering information
do <- "D4"
dom <- do[1]
k=8 

#Preparing data
longitude=rep(nc_lon,length(nc_lat))
latitude=rep(nc_lat,each=length(nc_lon))

Z <- list() 

load(file = paste("WardCluster_Dendrogram_","D4","_50km.Rdata",sep=""))
vector_cluster <- cutree(ward_clustering,k)

for(cp in 1:k){
  Z[[cp]] <-colMeans(var[which(vector_cluster==cp),],na.rm=T)
}

datitosERA5 <- data.frame(Longitude=rep(longitude,k),
                          Latitude=rep(latitude,k),
                          Z = unlist(Z),
                          CP = rep(1:k,each=length(unlist(Z))/k))
datitosERA5$CP <- factor(datitosERA5$CP,levels=c(1:k))

#Temporal cycle
calendario <- seq(as.Date("1950-01-01"),as.Date("2021-12-31"),by="day")
fechas <- calendario[format(calendario,"%m-%d") != "02-29"]
n_years <- length(fechas)/365

datos = vector_cluster
if(length(datos)!=length(fechas)){
  datos <- datos[which(calendario%in%fechas)]
  rm(calendario,fechas)
}

datos2 = data.frame(group=as.numeric( datos),
                    day=rep(1:365,n_years))

valueERA5 <- matrix(NA,365,k)
for(i in 1:365){
  for(j in 1:k){
    valueERA5[i,j] <- length(which(datos2$group==j & datos2$day==i))/n_years*100
  }
}

# avgERA5 <- matrix(NA,365,k)
# for(j in 1:k){
#   avgERA5[,j] <- avgmean(x=valueERA5[,j],ventana = 15)
# }

################################################################################
#Start working with models
files <- list.files(pattern="2014.nc",path = "/esarchive/scratch/molmo/esmvaltool_output/recipe_D4_20240503_055905/preproc/diagnostic_2/psl_d1/")
files2 <- files
files2[9] <- files[10]
files2[10] <- files[12]
files2[12] <- files[9]

files2[20] <- files[21]
files2[21] <- files[20]

files <- files2
filesW <- list.files(pattern="WardClustering",path = "/esarchive/scratch/molmo/output/D4/")
filesW2  <- intersect(filesW,list.files(pattern="_r",path = "/esarchive/scratch/molmo/output/D4/"))
filesW <- filesW[-which(filesW%in%filesW2)]
errors <- list()
gcms <- list()
meann <- list()

for(f in 1:length(files)){
  load(file = paste("/esarchive/scratch/molmo/output/D4/",filesW[f],sep=""))
  
  nc <- nc_open(paste("/esarchive/scratch/molmo/esmvaltool_output/recipe_D4_20240503_055905/preproc/diagnostic_2/psl_d1/",files[f],sep=""))
  nc_lon <- ncvar_get(nc,"lon")
  nc_lat <- ncvar_get(nc,"lat")
  nc_var <- ncvar_get(nc,"psl")
  var <- matrix(nc_var,dim(nc_var)[1]*dim(nc_var)[2],dim(nc_var)[3])
  
  var <- scale(t(var),scale=T,center=T)
  
  message("Reordering ",gcm)
  
  vector_cluster <- cutree(ward_clustering,k)
  #vector_cluster <- vector_cluster[which(calendario%in%fechas)]
  
  Zm <- list()
  for(cp in 1:k){
    Zm[[cp]] <- colMeans(var[which(vector_cluster==cp),],na.rm=T)
  }
  
  lon <- nc_lon
  for(i in 1:length(lon)){
    if(lon[i]>180){
      lon[i] <- lon[i]-360
    }
  }
  lat <- nc_lat    
  
  xmn <- min(lon) ; xmx <- max(lon) ; ymn <- min(lat) ; ymx <- max(lat)
  longitude=rep(lon,length(nc_lat))
  latitude=rep(seq(ymn,ymx,gridsize_x),each=length(nc_lon))
  
  datitos <- data.frame(Longitude=rep(longitude,k),
                        Latitude=rep(latitude,k),
                        Z = unlist(Zm),
                        CP = rep(1:k,each=length(unlist(Zm))/k))
  datitos$CP <- factor(datitos$CP,levels=c(1:k))
  
  #Ordeno las longitudes del GCM como en ERA5 para luego correlacionarlos / dist euclidea
  #Ahora viene el ordenamiento
  lonERA <- datitosERA5$Longitude[1:3500]
  latERA <- datitosERA5$Latitude[1:3500]
  
  # patronGCM <- list()
  # for(j in 1:k){
  #   patronGCM[[j]] <- NA
  #   for(i in 1:length(longitude)){
  #     patronGCM[[j]][i] <- datitos$Z[which(datitos$Longitude==lonERA[i] & datitos$Latitude==latERA[i] & datitos$CP==j)]
  #   }
  # }
  # 
  # #Matriz de distancias
  # distance <- matrix(NA,k,k)
  # for(i in 1:k){
  #   for(j in 1:k){
  #     distance[i,j] <- dist(rbind(Z[[i]],patronGCM[[j]]),method = "euclidean")
  #   }
  # }
  
  orden <- load.Rdata2(filename = paste("orden_CPs_",dom,"_",gcm,"_",k,".Rdata",sep=""),path = OUTPUTS)
  
  #Reordenamiento
  datos <- vector_cluster
  for(i in 1:k){
    datos[which(vector_cluster==i)] <- orden[i]
  }
  
  #Temporal cycle
  calendario <- seq(as.Date("1950-01-01"),as.Date("2014-12-31"),by="day")
  fechas <- calendario[format(calendario,"%m-%d") != "02-29"]
  n_years <- length(fechas)/365
  
  if(length(datos)!=length(fechas)){
    datos <- datos[which(calendario%in%fechas)]
    rm(calendario,fechas)
  }
  
  datos2 = data.frame(group=as.numeric( datos),
                      day=rep(1:365,n_years))
  
  value <- matrix(NA,365,k)
  for(i in 1:365){
    for(j in 1:k){
      value[i,j] <- length(which(datos2$group==j & datos2$day==i))/n_years*100
    }
  }
  
  # avg <- matrix(NA,365,k)
  # for(j in 1:k){
  #   avg[,j] <- avgmean(x=value[,j],ventana = 15)
  # }
  
  meann[[f]] <- colMeans(value)
  
  ErrorGCM <- matrix(NA,365,k)
  for(i in 1:365){
    for(j in 1:k){
      #ErrorGCM[i,j] <- (abs(valueERA5[i,j] - value[i,j])*100)#/mean(valueERA5[,j])
      ErrorGCM[i,j] <- (abs(valueERA5[i,j] - value[i,j])*100)#/valueERA5[i,j]
    }
  }
  
  errors[[f]] <- round(colMeans(ErrorGCM,na.rm = T),digits=1)/colSums(valueERA5)*100
  gcms[[f]] <- gcm
}

################################################################################
#Heatmaps ErrorCycle
data = data.frame(Error=unlist(errors),GCM=rep(unlist(gcms),each=k),CP=rep(c(1:k),length(unlist(gcms))))

 mean2 <- function(x,y){
#   pesos <- colSums(valueERA5)/365
#   sum <- (pesos*x)/length(x)
#   return(sum)
weighted.mean(x,w=y)
 }
 
meanError <- list()
for(f in 1:length(errors)){
  meanError[[f]] <- mean2(x=errors[[f]],y=meann[[f]])#meann[[f]]
}

data.mean <- data.frame(Error=as.numeric(meanError),GCM=unlist(gcms),CP=rep("Mean",length(unlist(gcms))))
#data.mean <- data.frame(Error=as.numeric(meanError),GCM=unlist(gcms),CP=rep("Mean",length(unlist(gcms))))
data.mean$Error <- round(data.mean$Error,digits=3)


#data$Error[which(data$Error > 20)] <- 20
data$GCM <- factor(data$GCM,levels=rev(unique(data$GCM)))
data <- rbind(data,data.mean)

data <- data[-which(data$GCM%in%c("NESM3","KACE-1-0-G")),]

p <- ggplot(data=data,aes(x=CP,y=GCM,fill=Error)) + geom_tile() + coord_cartesian(expand=F) + theme_bw()+
  scale_fill_gradientn(name="%",colours=rev(brewer.pal(11,"Spectral")),limits=c(0,75))#+ geom_text(data=subset(data,CP=="Mean"),aes(label=round(Error,digits=0)))
ggsave(p,filename=paste(OUTPUTS,"ErrorCycleRel_GCMs_",dom,"_","CPs.png",sep=""),device="png",
       dpi=600,width=5,height = 7)  

save(data,file="/esarchive/scratch/molmo/output/D4/ErrorCycleRel_D4.Rdata")

#Order (lowest to highest Error values)
ranking <- cbind(data.mean$GCM[order(data.mean$Error)],data.mean$Error[order(data.mean$Error)])
