library(loadeR)
library(transformeR)
library(ncdf4)
library(dplyr)

rm(list=ls()) ; gc()

#Tasmax
setwd("/esarchive/scratch/molmo/esmvaltool_output/recipe_surface2_intvar_20240629_084811/preproc/diagnostic_1/tasmax_d2/")

files <- list.files(pattern="2014.nc")
seasons <- list("Summer"=c(6:8),"Autumn"=(9:11),"Winter"=c(12,1,2),"Spring"=c(3:5))

trend <- list()
pv <- list()
gcm.tx <- list()
run.tx <- list()
for(i in seq_along(files)){
  message(paste(i,length(files),sep="/"))
  nc2 <- nc_open(files[i])
  run.tx[[i]] <- ncatt_get(nc2,0,)$variant_label
  gcm.tx[[i]] <- ncatt_get(nc2,0,)$source_id
  nc <- loadGridData(dataset=files[i],var="tasmax",aggr.m = "mean")
  
  trend[[i]] <- list()
  pv[[i]] <- list()
  
  for(s in seq_along(seasons)){
    temp <- aggregateGrid(nc,aggr.s = list(FUN=list("mean",na.rm=T),season=seasons[[s]]))
    trend[[i]][[s]] <- climatology(temp,clim.fun=list(FUN="trend.1D",dates=getRefDates(temp),return.pvalue=F))
    pv[[i]][[s]] <- climatology(temp,clim.fun=list(FUN="trend.1D",dates=getRefDates(temp),return.pvalue=T))
  }
  gc()
}

rm(nc,nc2,temp,gcm,i,s,run)
save.image("/esarchive/scratch/molmo/output/D4/intvar/tasmax_trends_intvar.Rdata")

#Precip
setwd("/esarchive/scratch/molmo/esmvaltool_output/recipe_surface2_intvar_20240629_084811/preproc/diagnostic_1/pr_d2/")

files <- list.files(pattern="2014.nc")
seasons <- list("Summer"=c(6:8),"Autumn"=(9:11),"Winter"=c(12,1,2),"Spring"=c(3:5))

trend <- list()
pv <- list()
gcm.pr <- list()
run.pr <- list()
for(i in seq_along(files)){
  message(paste(i,length(files),sep="/"))
  nc2 <- nc_open(files[i])
  run.pr[[i]] <- ncatt_get(nc2,0,)$variant_label
  gcm.pr[[i]] <- ncatt_get(nc2,0,)$source_id
  nc <- loadGridData(dataset=files[i],var="pr",aggr.m = "sum") %>% gridArithmetics(86400,operator = "*")
  
  trend[[i]] <- list()
  pv[[i]] <- list()
  
  for(s in seq_along(seasons)){
    temp <- aggregateGrid(nc,aggr.s = list(FUN=list("mean",na.rm=T),season=seasons[[s]]))
    trend[[i]][[s]] <- climatology(temp,clim.fun=list(FUN="trend.1D",dates=getRefDates(temp),return.pvalue=F))
    pv[[i]][[s]] <- climatology(temp,clim.fun=list(FUN="trend.1D",dates=getRefDates(temp),return.pvalue=T))
  }
  gc()
}

rm(nc,nc2,temp,gcm,i,s,run)
save.image("/esarchive/scratch/molmo/output/D4/intvar/precip_trends_intvar.Rdata")


#Tasmin
setwd("/esarchive/scratch/molmo/esmvaltool_output/recipe_surface2_intvar_20240629_084811/preproc/diagnostic_1/tasmin_d2/")

files <- list.files(pattern="2014.nc")
seasons <- list("Summer"=c(6:8),"Autumn"=(9:11),"Winter"=c(12,1,2),"Spring"=c(3:5))

trend <- list()
pv <- list()
gcm.tn <- list()
run.tn <- list()
for(i in seq_along(files)){
  message(paste(i,length(files),sep="/"))
  nc2 <- nc_open(files[i])
  run.tn[[i]] <- ncatt_get(nc2,0,)$variant_label
  gcm.tn[[i]] <- ncatt_get(nc2,0,)$source_id
  nc <- loadGridData(dataset=files[i],var="tasmin",aggr.m = "mean")
  
  trend[[i]] <- list()
  pv[[i]] <- list()
  
  for(s in seq_along(seasons)){
    temp <- aggregateGrid(nc,aggr.s = list(FUN=list("mean",na.rm=T),season=seasons[[s]]))
    trend[[i]][[s]] <- climatology(temp,clim.fun=list(FUN="trend.1D",dates=getRefDates(temp),return.pvalue=F))
    pv[[i]][[s]] <- climatology(temp,clim.fun=list(FUN="trend.1D",dates=getRefDates(temp),return.pvalue=T))
  }
  gc()
}

rm(nc,nc2,temp,gcm,i,s,run)
save.image("/esarchive/scratch/molmo/output/D4/intvar/tasmin_trends_intvar.Rdata")


#---------------------------------------------------------------------̣#
#Load ERA5 data to have a reference
seasons <- list("Summer"=c(6:8),"Autumn"=(9:11),"Winter"=c(12,1,2),"Spring"=c(3:5))

trend <- list()
pv <- list()
nc <- loadGridData(dataset="/esarchive/scratch/molmo/era5_tasmax_D2_5021.nc",var="tasmax",years=1950:2014)
temp <- loadGridData(dataset = "/esarchive/scratch/molmo/esmvaltool_output/recipe_surface2_intvar_20240629_084811/preproc/diagnostic_1/tasmax_d2/CMIP6_ACCESS-ESM1-5_day_historical_r10i1p1f1_tasmax_gn_1950-2014.nc",var="tasmax")
nc$Dates <- temp$Dates
rm(temp)  
  
nc <- aggregateGrid(nc,aggr.m=list(FUN="mean",na.rm=T))

trend <- list()
pv <- list()
  
for(s in seq_along(seasons)){
  temp <- aggregateGrid(nc,aggr.s = list(FUN=list("mean",na.rm=T),season=seasons[[s]]))
  trend[[s]] <- climatology(temp,clim.fun=list(FUN="trend.1D",dates=getRefDates(temp),return.pvalue=F))
  pv[[s]] <- climatology(temp,clim.fun=list(FUN="trend.1D",dates=getRefDates(temp),return.pvalue=T))
}
gc()  

temp <- NA
for(i in seq_along(trend)){
  temp[i] <- mean(trend[[i]]$Data)
}

ERA5 <- data.frame(TX_Trend=temp,Season=rep(names(seasons),each=length(temp)/4))

trend <- list()
pv <- list()
nc <- loadGridData(dataset="/esarchive/scratch/molmo/era5_prlr_D2_5021.nc",var="prlr",years=1950:2014)
temp <- loadGridData(dataset = "/esarchive/scratch/molmo/esmvaltool_output/recipe_surface2_intvar_20240629_084811/preproc/diagnostic_1/tasmax_d2/CMIP6_ACCESS-ESM1-5_day_historical_r10i1p1f1_tasmax_gn_1950-2014.nc",var="tasmax")
nc$Dates <- temp$Dates
rm(temp)  

nc <- aggregateGrid(nc,aggr.m=list(FUN="sum",na.rm=T))%>% gridArithmetics(86400,operator = "*")

trend <- list()
pv <- list()

for(s in seq_along(seasons)){
  temp <- aggregateGrid(nc,aggr.s = list(FUN=list("mean",na.rm=T),season=seasons[[s]]))
  trend[[s]] <- climatology(temp,clim.fun=list(FUN="trend.1D",dates=getRefDates(temp),return.pvalue=F))
  pv[[s]] <- climatology(temp,clim.fun=list(FUN="trend.1D",dates=getRefDates(temp),return.pvalue=T))
}
gc()  

temp <- NA
for(i in seq_along(trend)){
  temp[i] <- mean(trend[[i]]$Data)
}

ERA5$PR_Trend <- temp
save(ERA5,file="/esarchive/scratch/molmo/output/D4/intvar/ERA5_trend.Rdata")
