library(ggplot2)
library(RColorBrewer)

rm(list=ls()) ; gc()
#-----------------------------------#
#Load DISO data 
load("/esarchive/scratch/molmo/output/D4/DISO_PRECIP_intvar__D4_1x1_.Rdata")
DISO_PR <- datitos$DISO[which(datitos$CP=="Mean")]
GCM <- datitos$GCM[which(datitos$CP=="Mean")]
Run <- datitos$Run[which(datitos$CP=="Mean")]
gcmrun <- paste(GCM,Run)

load("/esarchive/scratch/molmo/output/D4/DISO_tasmin_intvar__D4_1x1_.Rdata")
DISO_TASMIN <- datitos$DISO[which(datitos$CP=="M")]
GCM <- datitos$GCM[which(datitos$CP=="M")]
Run <- datitos$Run[which(datitos$CP=="M")]
gcmrun2 <- paste(GCM,Run)


load("/esarchive/scratch/molmo/output/D4/DISO_TASMAX_intvar__D4_1x1_.Rdata")
DISO_TASMAX <- datitos$DISO[which(datitos$CP=="M")]
GCM <- datitos$GCM[which(datitos$CP=="M")]
Run <- datitos$Run[which(datitos$CP=="M")]
gcmrun3 <- paste(GCM,Run)

temp <- intersect(gcmrun2,gcmrun3)
temp <- intersect(temp,gcmrun)

DISO_PR <- DISO_PR[which(temp%in%gcmrun)]
DISO_TASMIN <- DISO_TASMIN[which(temp%in%gcmrun2)]
DISO_TASMAX <- DISO_TASMAX[which(temp%in%gcmrun3)]

#Load DISO data for SLP 
# load("/esarchive/scratch/molmo/output/D4/DISO_SLP.Rdata")
# DISO_SLP <- datitos$DISO[which(datitos$CP=="Mean")]
# GCM2 <- datitos$GCM[which(datitos$CP=="Mean")]
# DISO_SLP <- DISO_SLP[which(GCM2%in%GCM)]

#DISO_mean <- ((0.25*DISO_PR)+(0.25*DISO_TASMAX)+(0.25*DISO_TASMIN)+(0.25*DISO_SLP))
#DISO_mean <- ((0.75*DISO_PR)+(0.135*DISO_TASMAX)+(0.135*DISO_TASMIN))
DISO_mean <- ((0.5*DISO_PR)+(0.25*DISO_TASMAX)+(0.25*DISO_TASMIN))

#----------------_----------------#
#Load ErrorCycle data
load("/esarchive/scratch/molmo/output/D4/ErrorCycle_D4_IntVarRel.Rdata")
ErrorCycle <- data$Error[which(data$CP=="M")]
data <- data[which(data$CP=="M"),]
gcmrun4 <- paste(data$GCM,data$Run)

load("/esarchive/scratch/molmo/output/D4/DISO_PRECIP_intvar__D4_1x1_.Rdata")
GCM <- datitos$GCM[which(datitos$CP=="Mean")]
Run <- datitos$Run[which(datitos$CP=="Mean")]

datos <- data.frame(x=DISO_mean,y=ErrorCycle,GCM=GCM,Run=Run,Number=c(1:length(GCM)))
datos$GCM <- factor(datos$GCM,levels= unique(datos$GCM))#paste(c(1:length(GCM)),GCM))
datos$Run <- factor(datos$Run,levels=unique(datos$Run))#paste(c(1:length(Run)),Run))

#datos$y[which(datos$GCM=="26 NESM3")] <- 14.99
#datos$x[which(datos$x>0.75)] <- 0.75

datos2 <- datos
datos2$x <- datos2$x/mean(datos2$x)
datos2$y <- datos2$y/mean(datos2$y)

p <- ggplot(data=datos,aes(x=x,y=y,fill=GCM)) +
  geom_point(size=2,pch=21) + theme_bw() + coord_cartesian(expand = F)  +#geom_text(aes(x=x,y=y,label=Number),colour="black")+
  xlim(c(0,0.8)) + ylim(c(0,0.5)) + xlab("DISO mean") + ylab("ErrorCycle")+
  #scale_fill_manual(values=c(brewer.pal(name="Paired",n=11)[c(1,3,5,7,9,11)],brewer.pal(name="Set3",n=11)[c(1,8)])) + 
  scale_fill_manual(values=brewer.pal(n=8,"Set3"))+
  theme(legend.position = "bottom") 
#p <- ggplot(data=datos2,aes(x=x,y=y,colour=GCM)) +
 # geom_point(aes(size=1.2)) + theme_bw() + coord_cartesian(expand = F)  + geom_text(aes(x=x,y=y,label=Number),colour="black")+
  #xlim(c(0,1.6)) + ylim(c(0,1.6)) + xlab("DISO mean") + ylab("ErrorCycle")+
  #scale_colour_manual(values=rainbow(n=length(GCM))) + theme(legend.position = "bottom")

ggsave(p,filename="/esarchive/scratch/molmo/output/D4/dispersion_DISO_ErrorCycle_normalized_intvar.png",device="png",
       dpi=600,width=7,height = 7)   


##################################
#Euclidean distance
CalculateEuclideanDistance <- function(vect1, vect2) sqrt(sum((vect1 - vect2)^2)) 

manhattanDistance <- function(vect1, vect2){
  dist <- abs(vect1 - vect2)
  dist <- sum(dist)
  return(dist)
}

distancia <- NA
for(i in 1:dim(datos)[1]){
  distancia[i] <- CalculateEuclideanDistance(vect1 = c(datos$x[i]/mean(datos$x,na.rm=T),datos$y[i]/mean(datos$y,na.rm=T)),vect2=c(0,0))
}
datos$GCM[order(distancia)]

orden <-datos$GCM[order(distancia)]

save(orden,file="/esarchive/scratch/molmo/output/D4/orden_DISO_INTVAR.Rdata")

dist.data <- data.frame(Distance=distancia,GCM=GCM,Run=Run)

g <- ggplot(data=dist.data,aes(y=Distance,col=GCM)) + theme_bw()
ggsave(g,filename="dispersion_DISO_ErrorCycle_density.png",device="png",
          dpi=600,width=7,height = 7)   
# distancia <- NA
# for(i in 1:dim(datos)[1]){
#   distancia[i] <- manhattanDistance(vect1 = c(datos$x[i],datos$y[i]),vect2=c(0,0))
# }
# datos$GCM[order(distancia)]

dist <- matrix(NA,length(x),length(y))
for(i in 1:length(x)){
  for(j in 1:length(y)){
    dist[i,j] <- round(sqrt((x[i]^2)+(y[j]^2)),digits=3)
  }
}

data <- data.frame(x=rep(x,length(y)),y=rep(y,each=length(x)),dist=as.numeric(dist))
p30 <- quantile(data$dist,probs = 0.3) #34
p60 <- quantile(data$dist,probs=0.6) #46

p <- ggplot(data=datos,aes(x=x,y=y)) +
  geom_point(aes(size=1.2,colour=GCM)) + theme_bw() + coord_cartesian(expand = F)  + geom_text(aes(x=x,y=y,label=Number),colour="black")+
  xlim(c(0,0.75)) + ylim(c(0,15)) + xlab("DISO mean") + ylab("ErrorCycle")+
  scale_colour_manual(values=rainbow(n=length(GCM))) + theme(legend.position = "bottom")+
  geom_tile(data=data,aes(x=x,y=y,fill=dist))


which(dist==p60)


#------------------------#
#Catalonia
rm(list=ls()) ; gc()
#-----------------------------------#
#Load DISO data 
load("/esarchive/scratch/molmo/output/D4/DISO_PRECIP_D4_1x1__Cat.Rdata")
DISO_PR <- datitos$DISO[which(datitos$CP=="Mean")]
GCM <- datitos$GCM[which(datitos$CP=="Mean")]

load("/esarchive/scratch/molmo/output/D4/DISO_tasmin_D4_1x1__Cat.Rdata")
DISO_TASMIN <- datitos$DISO[which(datitos$CP=="Mean")]

load("/esarchive/scratch/molmo/output/D4/DISO_TASMAX_D4_1x1__Cat.Rdata")
DISO_TASMAX <- datitos$DISO[which(datitos$CP=="Mean")]


#Load DISO data for SLP 
# load("/esarchive/scratch/molmo/output/D4/DISO_SLP.Rdata")
# DISO_SLP <- datitos$DISO[which(datitos$CP=="Mean")]
# GCM2 <- datitos$GCM[which(datitos$CP=="Mean")]
# DISO_SLP <- DISO_SLP[which(GCM2%in%GCM)]

#DISO_mean <- ((0.25*DISO_PR)+(0.25*DISO_TASMAX)+(0.25*DISO_TASMIN)+(0.25*DISO_SLP))
#DISO_mean <- ((0.75*DISO_PR)+(0.135*DISO_TASMAX)+(0.135*DISO_TASMIN))
DISO_mean <- ((0.5*DISO_PR)+(0.25*DISO_TASMAX)+(0.25*DISO_TASMIN))

#----------------_----------------#
#Load ErrorCycle data
load("/esarchive/scratch/molmo/output/D4/ErrorCycle_D4.Rdata")
data <- data[which(data$GCM %in% GCM),]
ErrorCycle <- data$Error[which(data$CP=="Mean")]
DISO_mean <- DISO_mean[which(GCM %in% unique(data$GCM))]

GCM <- unique(data$GCM)
datos <- data.frame(x=DISO_mean,y=ErrorCycle,GCM=paste(c(1:length(GCM)),GCM),Number=c(1:length(GCM)))
datos$GCM <- factor(datos$GCM,levels=paste(c(1:length(GCM)),GCM))

#datos$y[which(datos$GCM=="26 NESM3")] <- 14.99
#datos$x[which(datos$x>0.75)] <- 0.75

p <- ggplot(data=datos,aes(x=x,y=y,colour=GCM)) +
  geom_point(aes(size=0.6)) + theme_bw() + coord_cartesian(expand = F)  + #geom_text(aes(x=x,y=y,label=Number),colour="black")+
  xlim(c(0,0.75)) + ylim(c(0,12)) + xlab("DISO mean") + ylab("ErrorCycle")+
  scale_colour_manual(values=rainbow(n=length(GCM))) + theme(legend.position = "bottom")

ggsave(p,filename="/esarchive/scratch/molmo/output/D4/dispersion_DISO_ErrorCycle_Cat.png",device="png",
       dpi=600,width=7,height = 7)   
